@@ -4,7 +4,7 @@ Attean - A Semantic Web Framework
 
 =head1 VERSION
 
-This document describes Attean version 0.006
+This document describes Attean version 0.008
 
 =head1 SYNOPSIS
 
@@ -41,7 +41,7 @@ Semantic Web (RDF and SPARQL) data.
 package Attean {
 	use v5.14;
 	use warnings;
-	our $VERSION	= '0.006';
+	our $VERSION	= '0.008';
 	use Attean::API;
 	
 	use Attean::Blank;
@@ -196,7 +196,7 @@ returns undef.
 				my @classes;
 				foreach my $class ($self->$method()) {
 					next unless (can_load( modules => { $class => 0 }));
-					push(@classes, $class) if ($class->does($role));
+					push(@classes, $class) if ($class->can('does') and $class->does($role));
 				}
 				return @classes;
 			};
