@@ -7,7 +7,7 @@ Attean::API::Binding - Name to term bindings
 
 =head1 VERSION
 
-This document describes Attean::API::Binding version 0.007
+This document describes Attean::API::Binding version 0.008
 
 =head1 DESCRIPTION
 
@@ -71,7 +71,7 @@ otherwise.
 
 use Type::Tiny::Role;
 
-package Attean::API::Binding 0.007 {
+package Attean::API::Binding 0.008 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
 	use List::MoreUtils qw(zip);
@@ -144,24 +144,46 @@ package Attean::API::Binding 0.007 {
 				$data{ $k }	= $v;
 			}
 		}
-		use Data::Dumper;
-		warn Dumper(\%data);
 		return $class->new( bindings => \%data );
 	}
 }
 
-package Attean::API::TripleOrQuadPattern 0.007 {
+package Attean::API::TripleOrQuadPattern 0.008 {
+	use Scalar::Util qw(blessed);
+	use Attean::RDF;
 	use Moo::Role;
+	use namespace::clean;
 
 	around BUILDARGS => sub {
 		my $orig 	= shift;
 		my $class	= shift;
-		if (scalar(@_) == 2) {
-			if ($_[0] eq 'bindings') {
-				return $class->$orig(%{ $_[1] });
+		my @args	= @_;
+		if (scalar(@args) == 0 or not(defined($_[0])) or blessed($args[0])) {
+			my @names	= $class->variables;
+			foreach my $i (0 .. $#names) {
+				my $k	= $names[$i];
+				my $v	= $args[$i];
+				unless (defined($v)) {
+					$args[$i]	= variable($k);
+				}
+			}
+			my %args;
+			@args{ $class->variables }	= @args;
+			return $class->$orig(%args);
+		} elsif (scalar(@args) == 2) {
+			if (defined($args[0]) and $args[0] eq 'bindings') {
+				return $class->$orig(%{ $args[1] });
 			}
 		}
-		return $class->$orig(@_);
+		
+		my %args	= @_;
+		foreach my $k ($class->variables) {
+			if (not(exists $args{$k}) or not($args{$k})) {
+				$args{$k}	= variable($k);
+			}
+		}
+		
+		return $class->$orig(%args);
 	};
 
 	sub apply_map {
@@ -171,16 +193,37 @@ package Attean::API::TripleOrQuadPattern 0.007 {
 		my %values	= map { $_ => $mapper->map($self->value($_)) } $self->variables;
 		return $class->new( %values );
 	}
+	
+	sub apply_statement {
+		my $self	= shift;
+		my $class	= ref($self);
+		my $bind	= shift;
+		my %data;
+		foreach my $k ($self->variables) {
+			my $v	= $self->value($k);
+			if ($v->does('Attean::API::Variable')) {
+				my $name	= $v->value;
+				my $replace	= $bind->value($name);
+				if (defined($replace) and blessed($replace)) {
+					$data{ $k }	= $replace;
+				} else {
+					$data{ $k }	= $v;
+				}
+			}
+		}
+		return Attean::Result->new( bindings => \%data );
+	}
 }
 
-package Attean::API::TripleOrQuad 0.007 {
+package Attean::API::TripleOrQuad 0.008 {
 	use Moo::Role;
 	with 'Attean::API::TripleOrQuadPattern';
 }
 
-package Attean::API::TriplePattern 0.007 {
+package Attean::API::TriplePattern 0.008 {
 	use Moo::Role;
 	use List::MoreUtils qw(zip);
+	use Scalar::Util qw(blessed);
 	use namespace::clean;
 	
 	sub variables { return qw(subject predicate object) }
@@ -213,6 +256,11 @@ package Attean::API::TriplePattern 0.007 {
 		return join(' ', map { $_->as_sparql } $self->values) . " .\n";
 	}
 	
+	sub apply_triple {
+		my $self	= shift;
+		return $self->apply_statement(@_);
+	}
+
 	requires 'subject';
 	requires 'predicate';
 	requires 'object';
@@ -220,7 +268,7 @@ package Attean::API::TriplePattern 0.007 {
 	with 'Attean::API::TripleOrQuadPattern', 'Attean::API::Binding';
 }
 
-package Attean::API::Triple 0.007 {
+package Attean::API::Triple 0.008 {
 	use Moo::Role;
 	
 	if ($ENV{ATTEAN_TYPECHECK}) {
@@ -255,7 +303,8 @@ package Attean::API::Triple 0.007 {
 	with 'Attean::API::TriplePattern', 'Attean::API::TripleOrQuad', 'Attean::API::Binding';
 }
 
-package Attean::API::QuadPattern 0.007 {
+package Attean::API::QuadPattern 0.008 {
+	use Scalar::Util qw(blessed);
 	use Moo::Role;
 	
 	sub variables { return qw(subject predicate object graph) }
@@ -274,6 +323,11 @@ package Attean::API::QuadPattern 0.007 {
 		return Attean::Quad->new($self->values);
 	}
 	
+	sub apply_quad {
+		my $self	= shift;
+		return $self->apply_statement(@_);
+	}
+
 	requires 'subject';
 	requires 'predicate';
 	requires 'object';
@@ -282,7 +336,7 @@ package Attean::API::QuadPattern 0.007 {
 	with 'Attean::API::TripleOrQuadPattern', 'Attean::API::Binding';
 }
 
-package Attean::API::Quad 0.007 {
+package Attean::API::Quad 0.008 {
 	use Moo::Role;
 	
 	if ($ENV{ATTEAN_TYPECHECK}) {
@@ -309,7 +363,7 @@ package Attean::API::Quad 0.007 {
 }
 
 
-package Attean::API::Result 0.007 {
+package Attean::API::Result 0.008 {
 	use Moo::Role;
 	use Scalar::Util qw(refaddr);
 	use namespace::clean;
