@@ -7,7 +7,7 @@ Attean::API::Term - RDF Terms
 
 =head1 VERSION
 
-This document describes Attean::API::Term version 0.006
+This document describes Attean::API::Term version 0.008
 
 =head1 DESCRIPTION
 
@@ -43,7 +43,7 @@ Returns a string serialization of the term.
 
 =cut
 
-package Attean::API::Term 0.006 {
+package Attean::API::Term 0.008 {
 	use Moo::Role;
 	
 	with 'Attean::API::TermOrVariable', 'Attean::API::ResultOrTerm';
@@ -104,7 +104,7 @@ package Attean::API::Term 0.006 {
 	}
 }
 
-package Attean::API::Literal 0.006 {
+package Attean::API::Literal 0.008 {
 	use Moo::Role;
 	use IRI;
 	use Scalar::Util qw(blessed);
@@ -216,9 +216,21 @@ package Attean::API::Literal 0.006 {
 			}
 		}
 	}
+
+	around as_sparql => sub {
+		my $orig	= shift;
+		my $self	= shift;
+		my $s		= $self->$orig(@_);
+		
+		if ($s =~ m[^"(true|false)"\^\^<http://www[.]w3[.]org/2001/XMLSchema#boolean>$]) {
+			return $1;
+		}
+		
+		return $s;
+	};
 }
 
-package Attean::API::DateTimeLiteral 0.006 {
+package Attean::API::DateTimeLiteral 0.008 {
 	use Moo::Role;
 	use DateTime::Format::W3CDTF;
 	use namespace::clean;
@@ -229,12 +241,12 @@ package Attean::API::DateTimeLiteral 0.006 {
 	}
 }
 
-package Attean::API::CanonicalizingLiteral 0.006 {
+package Attean::API::CanonicalizingLiteral 0.008 {
 	use Moo::Role;
 	requires 'canonicalized_term';
 }
 
-package Attean::API::NumericLiteral 0.006 {
+package Attean::API::NumericLiteral 0.008 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed looks_like_number);
 	use namespace::clean;
@@ -425,7 +437,7 @@ package Attean::API::NumericLiteral 0.006 {
 	with 'Attean::API::Literal', 'Attean::API::CanonicalizingLiteral';
 }
 
-package Attean::API::Blank 0.006 {
+package Attean::API::Blank 0.008 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
 	use namespace::clean;
@@ -441,7 +453,7 @@ package Attean::API::Blank 0.006 {
 	}
 }
 
-package Attean::API::IRI 0.006 {
+package Attean::API::IRI 0.008 {
 	use Moo::Role;
 	use IRI;
 	use Scalar::Util qw(blessed);
@@ -462,6 +474,17 @@ package Attean::API::IRI 0.006 {
 		my $self	= shift;
 		return sprintf('<%s>', $self->__ntriples_string);
 	}
+
+	around as_sparql => sub {
+		my $orig	= shift;
+		my $self	= shift;
+		
+		if ($self->value eq 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type') {
+			return 'a';
+		}
+
+		return $self->$orig(@_);
+	};
 }
 
 1;
