@@ -7,7 +7,7 @@ Attean::Plan - Representation of SPARQL query plan operators
 
 =head1 VERSION
 
-This document describes Attean::Plan version 0.007
+This document describes Attean::Plan version 0.008
 
 =head1 SYNOPSIS
 
@@ -31,7 +31,7 @@ Evaluates a quad pattern against the model.
 
 =cut
 
-package Attean::Plan::Quad 0.007 {
+package Attean::Plan::Quad 0.008 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf ArrayRef);
 
@@ -81,14 +81,9 @@ Evaluates a join (natural-, anti-, or left-) using a nested loop.
 
 =cut
 
-package Attean::Plan::NestedLoopJoin 0.007 {
+package Attean::Plan::NestedLoopJoin 0.008 {
 	use Moo;
-	use Types::Standard qw(ArrayRef Str Bool ConsumerOf);
-	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
-	has 'join_variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
-	has 'left' => (is => 'ro', isa => Bool, default => 0);
-	has 'anti' => (is => 'ro', isa => Bool, default => 0);
-	has 'expression' => (is => 'ro', isa => ConsumerOf['Attean::API::Expression'], required => 0, default => sub { Attean::ValueExpression->new( value => Attean::Literal->true ) });
+	with 'Attean::API::Plan::Join';
 	sub plan_as_string {
 		my $self	= shift;
 		if ($self->left) {
@@ -144,14 +139,9 @@ Evaluates a join (natural-, anti-, or left-) using a hash join.
 
 =cut
 
-package Attean::Plan::HashJoin 0.007 {
+package Attean::Plan::HashJoin 0.008 {
 	use Moo;
-	use Types::Standard qw(ArrayRef Str ConsumerOf Bool);
-	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
-	has 'join_variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
-	has 'left' => (is => 'ro', isa => Bool, default => 0);
-	has 'anti' => (is => 'ro', isa => Bool, default => 0);
-	has 'expression' => (is => 'ro', isa => ConsumerOf['Attean::API::Expression'], required => 0, default => sub { Attean::ValueExpression->new( value => Attean::Literal->true ) });
+	with 'Attean::API::Plan::Join';
 	sub plan_as_string {
 		my $self	= shift;
 		my $name;
@@ -220,12 +210,28 @@ named variable binding.
 
 =cut
 
-package Attean::Plan::EBVFilter 0.007 {
+package Attean::Plan::EBVFilter 0.008 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str ConsumerOf);
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
 	has 'variable' => (is => 'ro', isa => Str, required => 1);
+
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub plan_as_string {
 		my $self	= shift;
 		return sprintf('EBVFilter { ?%s }', $self->variable);
@@ -256,7 +262,7 @@ ordering.
 
 =cut
 
-package Attean::Plan::Merge 0.007 {
+package Attean::Plan::Merge 0.008 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str ArrayRef ConsumerOf);
@@ -264,6 +270,21 @@ package Attean::Plan::Merge 0.007 {
 	has 'variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
 	sub plan_as_string { return 'Merge' }
 	
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
@@ -280,12 +301,27 @@ Evaluates a set of sub-plans, returning the union of results.
 
 =cut
 
-package Attean::Plan::Union 0.007 {
+package Attean::Plan::Union 0.008 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
 	sub plan_as_string { return 'Union' }
 	
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
@@ -322,9 +358,10 @@ expressions, binding the produced values to new variables.
 
 =cut
 
-package Attean::Plan::Extend 0.007 {
+package Attean::Plan::Extend 0.008 {
 	use Moo;
 	use Scalar::Util qw(blessed);
+	use List::MoreUtils qw(uniq);
 	use Types::Standard qw(ConsumerOf HashRef);
 	use namespace::clean;
 
@@ -337,7 +374,21 @@ package Attean::Plan::Extend 0.007 {
 	}
 	sub tree_attributes { return qw(variable expression) };
 	
+	sub BUILDARGS {
+		my $class		= shift;
+		my %args		= @_;
+		my $exprs		= $args{ expressions };
+		my @vars		= map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @evars		= (@vars, keys %$exprs);
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= [@evars];
 
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub evaluate_expression {
 		my $self	= shift;
 		my $model	= shift;
@@ -434,11 +485,26 @@ hash of already-seen results.
 
 =cut
 
-package Attean::Plan::HashDistinct 0.007 {
+package Attean::Plan::HashDistinct 0.008 {
 	use Moo;
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
 	sub plan_as_string { return 'HashDistinct' }
 	
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
@@ -458,11 +524,26 @@ filtering out sequential duplicates.
 
 =cut
 
-package Attean::Plan::Unique 0.007 {
+package Attean::Plan::Unique 0.008 {
 	use Moo;
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
 	sub plan_as_string { return 'Unique' }
 	
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
@@ -489,12 +570,28 @@ number of results ("offset") and limiting the total number of returned results
 
 =cut
 
-package Attean::Plan::Slice 0.007 {
+package Attean::Plan::Slice 0.008 {
 	use Moo;
 	use Types::Standard qw(Int);
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
 	has 'limit' => (is => 'ro', isa => Int, default => -1);
 	has 'offset' => (is => 'ro', isa => Int, default => 0);
+
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub plan_as_string {
 		my $self	= shift;
 		my @str;
@@ -525,11 +622,25 @@ of variable bindings in each result.
 
 =cut
 
-package Attean::Plan::Project 0.007 {
+package Attean::Plan::Project 0.008 {
 	use Moo;
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	has 'variables' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']], required => 1);
+
+	sub BUILDARGS {
+		my $class		= shift;
+		my %args		= @_;
+		my @vars		= map { $_->value } @{ $args{variables} };
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub plan_as_string {
 		my $self	= shift;
 		return sprintf('Project { %s }', join(' ', map { '?' . $_->value } @{ $self->variables }));
@@ -559,7 +670,7 @@ sorting is applied.
 
 =cut
 
-package Attean::Plan::OrderBy 0.007 {
+package Attean::Plan::OrderBy 0.008 {
 	use Moo;
 	use Types::Standard qw(HashRef ArrayRef InstanceOf Bool Str);
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
@@ -573,6 +684,21 @@ package Attean::Plan::OrderBy 0.007 {
 		return sprintf('Order { %s }', join(', ', @strings));
 	}
 	
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
@@ -603,13 +729,89 @@ package Attean::Plan::OrderBy 0.007 {
 	}
 }
 
+package Attean::Plan::HeapSort 0.008 {
+	use Moo;
+	use Array::Heap;
+	use Types::Standard qw(Int HashRef ArrayRef InstanceOf Bool Str);
+	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+	has 'limit' 	=> (is => 'ro', isa => Int, default => -1);
+	has 'variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
+	has 'ascending' => (is => 'ro', isa => HashRef[Bool], required => 1);
+
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
+	sub plan_as_string {
+		my $self		= shift;
+		my @vars		= @{ $self->variables };
+		my $ascending	= $self->ascending;
+		my @cmpstrs	= map { sprintf('%s(?%s)', ($ascending->{$_} ? 'ASC' : 'DESC'), $_) } @vars;
+
+		my @str;
+		push(@str, "Limit=" . $self->limit);
+		push(@str, "Order=" . join(', ', @cmpstrs));
+		return sprintf('HeapSort { %s }', join(' ', @str));
+	}
+	
+	sub impl {
+		my $self	= shift;
+		my $model	= shift;
+		my $vars	= $self->variables;
+		my $ascending	= $self->ascending;
+		my ($impl)	= map { $_->impl($model) } @{ $self->children };
+		my $limit	= $self->limit;
+		my $cmp	= sub {
+			my $a	= shift;
+			my $b	= shift;
+			my ($ar, $avalues)	= ($a, [map { $a->value($_) } @$vars]);
+			my ($br, $bvalues)	= ($b, [map { $b->value($_) } @$vars]);
+			my $c	= 0;
+			foreach my $i (0 .. $#{ $vars }) {
+				my $ascending	= $ascending->{ $vars->[$i] };
+				my ($av, $bv)	= map { $_->[$i] } ($avalues, $bvalues);
+				$c		= $av ? $av->compare($bv) : 1;
+				$c		*= -1 unless ($ascending);
+				last unless ($c == 0);
+			}
+			return -$c;	# negate the sort order so that we maintain the results we do want to keep in the heap
+		};
+		return sub {
+			my $iter	= $impl->();
+			my @heap;
+			while (my $r = $iter->next) {
+				push_heap_cmp(sub { $cmp->($a, $b) }, @heap, $r);
+				if (scalar(@heap) > $limit) {
+					pop_heap_cmp(sub { $cmp->($a, $b) }, @heap);
+				}
+			}
+			my @rows;
+			while (my $r = pop_heap_cmp(sub { $cmp->($a, $b) }, @heap)) {
+				unshift(@rows, $r);
+			}
+			return Attean::ListIterator->new( values => \@rows, item_type => $iter->item_type);
+		}
+	}
+}
+
 =item * L<Attean::Plan::Service>
 
 Evaluates a SPARQL query against a remove endpoint.
 
 =cut
 
-package Attean::Plan::Service 0.007 {
+package Attean::Plan::Service 0.008 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf Bool Str);
 	sub plan_as_string {
@@ -637,7 +839,7 @@ Returns a constant set of results.
 
 =cut
 
-package Attean::Plan::Table 0.007 {
+package Attean::Plan::Table 0.008 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -647,6 +849,19 @@ package Attean::Plan::Table 0.007 {
 	sub tree_attributes { return qw(variables rows) };
 	sub plan_as_string { return 'Table' }
 	
+	sub BUILDARGS {
+		my $class		= shift;
+		my %args		= @_;
+		my @vars		= map { $_->value } @{ $args{variables} };
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
@@ -667,7 +882,7 @@ results were produced by evaluating the sub-plan.
 
 =cut
 
-package Attean::Plan::Exists 0.007 {
+package Attean::Plan::Exists 0.008 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -677,6 +892,21 @@ package Attean::Plan::Exists 0.007 {
 	sub tree_attributes { return qw(variables rows) };
 	sub plan_as_string { return 'Exists' }
 	
+	sub BUILDARGS {
+		# TODO: this code is repeated in several plan classes; figure out a way to share it.
+		my $class		= shift;
+		my %args		= @_;
+		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
+		my @vars		= keys %vars;
+		
+		if (exists $args{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		$args{in_scope_variables}	= \@vars;
+
+		return $class->SUPER::BUILDARGS(%args);
+	}
+	
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
@@ -693,7 +923,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Ask 0.007 {
+# package Attean::Algebra::Ask 0.008 {
 # 	use Moo;
 # 	sub in_scope_variables { return; }
 # 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
@@ -715,7 +945,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Construct 0.007 {
+# package Attean::Algebra::Construct 0.008 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
@@ -742,7 +972,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Path 0.007 {
+# package Attean::Algebra::Path 0.008 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	sub in_scope_variables {
@@ -780,7 +1010,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::NegatedPropertySet 0.007 {
+# package Attean::Algebra::NegatedPropertySet 0.008 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -801,7 +1031,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::PredicatePath 0.007 {
+# package Attean::Algebra::PredicatePath 0.008 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -825,7 +1055,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::InversePath 0.007 {
+# package Attean::Algebra::InversePath 0.008 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -841,7 +1071,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::SequencePath 0.007 {
+# package Attean::Algebra::SequencePath 0.008 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "/" }
@@ -856,7 +1086,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::AlternativePath 0.007 {
+# package Attean::Algebra::AlternativePath 0.008 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "|" }
@@ -871,7 +1101,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrMorePath 0.007 {
+# package Attean::Algebra::ZeroOrMorePath 0.008 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -887,7 +1117,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::OneOrMorePath 0.007 {
+# package Attean::Algebra::OneOrMorePath 0.008 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -903,7 +1133,7 @@ package Attean::Plan::Exists 0.007 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrOnePath 0.007 {
+# package Attean::Algebra::ZeroOrOnePath 0.008 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
